import React, { useState } from "react";
import UheppHist from "./UheppHist.jsx";
import { objfilter, objmap } from "../helpers/uhepp.js";

const noVariationUsed = (uhepp) => {
  const main_names = uhepp.stacks.map(stack => stack.content.map(item => item.yields)).flat()
  const num_names = uhepp.ratio.map(item => item.numerator).flat()
  const den_names = uhepp.ratio.map(item => item.denominator).flat()
  const names = [main_names, den_names, num_names].flat()
    
  const varied_names = names.filter(name => (name && name.indexOf("/") != -1))
  return varied_names.length == 0
}

const variationList = (uhepp) => {
  return Array.from(new Set(Object.entries(uhepp.yields).map(([key, item]) => [
      ...Object.keys(item.var_up || {}),
      ...Object.keys(item.var_down || {})
  ]).flat())).sort()
}

const varstyle = (updown) => {
  if (updown == "up") {
    return {color: "#ff0000"}
  } if (updown == "down") {
    return {color: "#0000ff"}
  }  else {
    return {}
  }
}

const makeVariationStacks = (uhepp, stackId, variation, updown) => {
  const stack = uhepp.stacks[stackId]
  return  [{
            type: "step", 
            content: [{
              style: varstyle(updown),
              "label": updown,
              "yield": stack.content.map(item => 
                item.yield.map(name => `${name}/${variation}/${updown}`)
              ).flat()
            }]
          }]
}

const makeVariationRatio = (uhepp, stackId, variation, updown) => {
  const stack = uhepp.stacks[stackId]
  return  [{
            type: "step", 
            style: varstyle(updown),
            numerator: stack.content.map(item => 
                item.yield.map(name => `${name}/${variation}/${updown}`)
              ).flat(),
            denominator: stack.content.map(item => 
                item.yield.map(name => `${name}`)
              ).flat()
          }]
}

const UheppHistUIWithSyst = ({
  width,
  height,
  uhepp,
  onEnvChange,
  onEnvStackChange,
  onReset,
  isVariationReady,
  variations,
  origStacks,
  envName,
  envId,
}) => {
  const [include_underflow, setUnderflow] = useState(!!uhepp.bins.include_underflow)
  const [include_overflow, setOverflow] = useState(!!uhepp.bins.include_overflow)
  const [rebin, setRebin] = useState(uhepp.bins.rebin || uhepp.bins.edges)
  const [density_width, setDensity] = useState(uhepp.bins.density_width)

  const reset = () => {
    setUnderflow(!!uhepp.bins.include_underflow)
    setOverflow(!!uhepp.bins.include_overflow)
    setRebin(uhepp.bins.rebin || uhepp.bins.edges)
    setDensity(uhepp.bins.density_width)
    onReset()
  }

  const handleRebin = (e) => {
    let values = Array.from(e.target.selectedOptions, option => parseFloat(option.value))
    setRebin(values)
  }


  const handleUnderflow = (e) => {
    let value = e.target.checked
    setUnderflow(value)
  }
  const handleOverflow = (e) => {
    let value = e.target.checked
    setOverflow(value)
  }
  const handleDensityWidth = (e) => {
    let value = parseFloat(e.target.value)
    setDensity(value)
  }

  const uhepp_derived = Object.assign({}, uhepp,
      {bins: Object.assign({}, uhepp.bins, {
          rebin: rebin.length > 1 ? rebin : (uhepp.bins.rebin),
          include_underflow,
          include_overflow,
          density_width,
      })})

  return <>
		<div>{ Object.entries(uhepp.metadata.tags).map(([key,  value], i) => 
				value == null ?
				<span key={i} className="badge badge-pill badge-primary mx-1">{key}</span> :
				<span key={i} className="badge-pair mx-1">
					<span className="badge badge-pill badge-primary">{key}</span>
					<span className="badge badge-pill badge-secondary">{value}</span>
				</span>
				) }</div>
    <UheppHist width={width} height={height} uhepp={uhepp_derived} />

		<ul className="nav nav-tabs" id="view-options" role="tablist">
			<li className="nav-item">
				<a className="nav-link active" id="info-tab" data-toggle="tab" href="#info" role="tab" aria-controls="info" aria-selected="true">Info</a>
			</li>
			<li className="nav-item">
				<a className="nav-link" id="binning-tab" data-toggle="tab" href="#binning" role="tab" aria-controls="binning" aria-selected="true">Binning</a>
			</li>
			{ <li className="nav-item">
				<a className="nav-link" id="variations-tab" data-toggle="tab" href="#variations" role="tab" aria-controls="variations" aria-selected="false">Variations</a>
			</li> }
			<li className="nav-item">
				<a className="nav-link" id="reset-tab" data-toggle="tab" href="#reset" role="tab" aria-controls="reset" aria-selected="false">Reset</a>
			</li>
		</ul>

		<div className="tab-content" id="view-options-content">
			<div className="tab-pane show active p-3" id="info" role="tabpanel" aria-labelledby="binning-tab">
				<dl>
					<dt>Author</dt>
					<dd>{ uhepp.metadata.author || <i>None</i>}</dd>
					<dt>Date</dt>
					<dd>{ uhepp.metadata.date ? (new Date(uhepp.metadata.date)).toString() : <i>None</i>}</dd>
					<dt>Producer</dt>
					<dd>{ uhepp.metadata.producer || <i>None</i>}</dd>
					<dt>Code revision</dt>
					<dd>{ uhepp.metadata.code_revision || <i>None</i>}</dd>
					<dt>Event selection</dt>
					<dd>{ uhepp.metadata.event_selection || <i>None</i>}</dd>
					<dt>Total integrated luminosity</dt>
					<dd>{ uhepp.metadata.lumi_ifb ?
								<span>{ uhepp.metadata.lumi_ifb} fb<sup>-1</sup></span> : <i>None</i>}</dd>
					<dt>Center-of-mass energy</dt>
					<dd>{ uhepp.metadata.Ecm_TeV ?
								uhepp.metadata.Ecm_TeV + " TeV" : <i>None</i>}</dd>
					<dt>Tags</dt>
					<dd>{ Object.entries(uhepp.metadata.tags).map(([key,  value], i) => 
							value == null ?
							<span key={i} className="badge badge-pill badge-primary mx-1">{key}</span> :
							<span key={i} className="badge-pair mx-1">
								<span className="badge badge-pill badge-primary">{key}</span>
								<span className="badge badge-pill badge-secondary">{value}</span>
							</span>
							) }</dd>
				</dl>
			</div>
			<div className="tab-pane p-3" id="binning" role="tabpanel" aria-labelledby="binning-tab">
				<form>
					<div className="form-group">
						<label htmlFor="rebin">Bin edges</label>
						<select multiple size={10} className="form-control" id="rebin" onChange={(e) => handleRebin(e)}
                value={rebin}>
							{uhepp.bins.edges.map((v, i) => 
								<option value={v} key={i}>{v}</option>
							)}
						</select>
					</div>

					<div className="form-check">
						<input className="form-check-input" type="checkbox"
							checked={include_underflow == true} id="checkUnderflow"
							onChange={e => handleUnderflow(e)} />
						<label className="form-check-label" htmlFor="checkUnderflow">
							Include underflow events in first bin
						</label>
					</div>

					<div className="form-check">
						<input className="form-check-input" type="checkbox"
							checked={include_overflow == true} id="checkOverflow" 
							onChange={e => handleOverflow(e)} />
						<label className="form-check-label" htmlFor="checkOverflow">
							Include overflow events in last bin
						</label>
					</div>

					<div className="form-group">
						<label htmlFor="densityWidth">Normalize yields to</label>
						<div className="input-group">
							<input type="number" className="form-control" id="densityWidth"
								onChange={e => handleDensityWidth(e)}
								value={density_width || "0"} />
              { uhepp.variable.unit &&
                <div className="input-group-append">
                  <span className="input-group-text">{uhepp.variable.unit}</span>
                </div>
              }
						</div>
					</div>
				</form>
			</div>
			<div className="tab-pane p-3" id="variations" role="tabpanel" aria-labelledby="variations-tab">
        { !isVariationReady && 
          <p>To use the variation feature, you must add variation data to
          the <code>yields</code> object. Please note that the stacks must not use
          any varied yield in the default view.</p> }
        { isVariationReady && 
        <form>
					<div className="form-group">
						<label htmlFor="envelop">Add envelop of </label>
						<select value={envName} className="form-control" id="envelop" onChange={(e) => onEnvChange(e)}>
              { [
                <option value="NOMINAL" key={"nonminal"}>Nominal</option>,
                ...variations.map((name, i) => <option value={name} key={i}>{name}</option>)
                ]
              }
            </select>
          </div>
					<div className="form-group">
						<label htmlFor="env-stack">for</label>
						<select value={envId} className="form-control" id="env-stack" onChange={(e) => onEnvStackChange(e)}>
              { origStacks.map((stack, i) => <option value={i} key={i}>Stack {i} ({stack.content.length} items, {stack.type})</option>) }
            </select>
          </div>
        </form>
        }
			</div>
			<div className="tab-pane p-3" id="reset" role="tabpanel" aria-labelledby="reset-tab">
        <p>Reset all view modifications. Pull the plot to make permanent
        changes.</p>
				<button className="btn btn-secondary m-1" type="button" onClick={() => reset()}>
					Reset view
				</button>
			</div>
		</div>
  </>
}

const UheppHistUI = ({width, height, uhepp}) => {
  const [envId, setEnvId] = useState(0)
  const [envName, setEnvName] = useState("NOMINAL")

  const reset = () => {
    setEnvId(0)
    setEnvName("NOMINAL")
  }
  const handleEnvelop = (e) => {
    let variationName = e.target.value
    setEnvName(variationName)
  }
  const handleEnvStack = (e) => {
    let stackId = parseInt(e.target.value)
    setEnvId(stackId)
  }

  const variations = variationList(uhepp)
  const isVariationReady = noVariationUsed(uhepp) && (variations.length > 0)

  const main_names = uhepp.stacks.map(stack => stack.content.map(item => item.yields)).flat()
  const num_names = uhepp.ratio.map(item => item.numerator).flat()
  const den_names = uhepp.ratio.map(item => item.denominator).flat()
  const all_names = [...main_names, ...num_names, ...den_names]

  const filter_var = (var_updown) => objfilter(var_updown, (value, key) => ((all_names.indexOf(key) != -1) || key.indexOf(envName) != -1))
  const pruned = Object.assign({}, uhepp,
    {"yields": objmap(uhepp.yields, y => Object.assign({}, y, {
      "var_up": filter_var(y.var_up || {}), 
      "var_down": filter_var(y.var_down || {}), 
    }))}
  )

  const pruned_env = envName == "NOMINAL" ? pruned : Object.assign({}, pruned,
        {stacks: [
          ...pruned.stacks,
          ...makeVariationStacks(pruned, envId, envName, "up"),
          ...makeVariationStacks(pruned, envId, envName, "down"),
        ],
        ratio: [
          ...makeVariationRatio(pruned, envId, envName, "up"),
          ...makeVariationRatio(pruned, envId, envName, "down"),
          ...pruned.ratio,
        ]},
      )

  return <UheppHistUIWithSyst
            width={width}
            height={height}
            uhepp={pruned_env}
            onEnvChange={e => handleEnvelop(e)}
            onEnvStackChange={e => handleEnvStack(e)}
            onReset={() => reset()}
            envName={envName}
            envId={envId}
            origStacks={pruned.stacks}
            isVariationReady={isVariationReady}
            variations={variations} />
}
export default UheppHistUI;
